<?php

namespace Subsitefunctions;


class ClassLoader
{

    protected $prefixes = array();


    public function register()
    {
        spl_autoload_register(array($this, 'loadClass'));
    }

    public function unregister()
    {
        spl_autoload_unregister(array($this, 'loadClass'));
    }

    /**
     * Adds a base directory for a namespace prefix.
     *
     * @param string $prefix The namespace prefix.
     * @param string $baseDir A base directory for class files in the
     * namespace.
     * @param bool $prepend If true, prepend the base directory to the stack
     * instead of appending it; this causes it to be searched first rather
     * than last.
     * @return void
     */
    public function addNamespace($prefix, $baseDir, $prepend = false)
    {
        $prefix = trim($prefix, '\\') . '\\';

        $baseDir = rtrim($baseDir, DIRECTORY_SEPARATOR) . '/';


        if (isset($this->prefixes[$prefix]) === false)
        {
            $this->prefixes[$prefix] = array();
        }


        if ($prepend)
        {
            array_unshift($this->prefixes[$prefix], $baseDir);
        } else
        {
            array_push($this->prefixes[$prefix], $baseDir);
        }
    }

    /**
     * Loads the class file for a given class name.
     *
     * @param string $class The fully-qualified class name.
     * @return mixed The mapped file name on success, or boolean false on
     * failure.
     */
    public function loadClass($class)
    {
        $prefix = $class;
        while (false !== $pos = strrpos($prefix, '\\'))
        {
            $prefix = substr($class, 0, $pos + 1);
            $relative_class = substr($class, $pos + 1);

            $mapped_file = $this->loadMappedFile($prefix, $relative_class);
            if ($mapped_file)
            {
                return $mapped_file;
            }

            $prefix = rtrim($prefix, '\\');
        }
        return false;
    }


    protected function loadMappedFile($prefix, $relative_class)
    {
        if (isset($this->prefixes[$prefix]) === false)
        {
            return false;
        }
        foreach ($this->prefixes[$prefix] as $base_dir)
        {
            $file = $base_dir
                    . str_replace('\\', '/', $relative_class)
                    . '.php';


            if ($this->requireFile($file))
            {
                return $file;
            }
        }
        return false;
    }


    protected function requireFile($file)
    {
        if (file_exists($file))
        {
            require $file;
            return true;
        }
        return false;
    }

}
